
List do(
    docSlot("sum", "Returns the sum of the items.")
    sum := method(s := 0; self foreach(v, s = s + v); s)

    docSlot("average", "Returns the average of the items.")
    average := method(self sum / self size)

    docSlot("shuffle", "Randomizes the order of the elements in the receiver. Returns self.")
    shuffle := method(for(i, 0, size - 1, swapIndices(i, Random value(i, size) floor)))

    docSlot("anyOne", "Returns a random element of the receiver or nil if the receiver is empty.")
    anyOne := method(at(Random value(0, size) floor))
 
    docSlot("removeFirst", "Returns the first item and removes it from the list. nil is returned if the list is empty.")
    removeFirst := method(if(self size != 0, removeAt(0), nil))

    docSlot("removeLast", "Returns the last item and removes it from the list. nil is returned if the list is empty.")
    removeLast := method(self pop)

    docSlot("join(optionalString)", "Returns a Sequence of the concatenated items with optionalString between each item or simply the concatenation of the items if no optionalString is supplied.")
    join := method(arg,
		s := Sequence clone
		if(arg) then(
			max := self size - 1
			self foreach(i, v, s appendSeq(v); if(i != max, s appendSeq(arg)))
		) else(
			self foreach(v, s appendSeq(v))
		)
		s
    )

    docSlot("insertAfter(item, afterItem)", "Inserts item after first occurance of afterItem and returns self. If afterItem is not found, item is appended to the end of the list.")
    insertAfter := method(item, afterItem,
       i := self indexOf(afterItem) 
       if(i, self atInsert(i + 1, item), self append(item))
       self
    )

    docSlot("insertBefore(item, beforeItem)", "Inserts item before first occurance of beforeItem or to the end of the list if beforeItem is not found. Returns self.")
    insertBefore := method(item, beforeItem,
       i := self indexOf(beforeItem)
       if(i, self atInsert(i, item), self append(item))
       self
    )

    docSlot("insertAt(item, index)", "Inserts item at the specified index. Raises an exception if the index is out of bounds. Returns self.")
    insertAt := method(item, index, self atInsert(index, item))
    
    asString := method(
    	s := "list(" asMutable
    	self foreach(i, v, 
    		s appendSeq(getSlot("v") asSimpleString)
    		if (i != self size - 1, s appendSeq(", "))
    	)
     	s appendSeq(")")
     )
     
     max := method(
     	m := call argAt(0) 
     	obj := self first
     	self foreach(o, 
     		v1 := if(m, o doMessage(m, call sender), o)
     		v2 := if(m, obj doMessage(m, call sender), obj)
     		if(v1 > v2, obj := o)
     	)
     	obj
     )
     
     min := method(
     	m := call argAt(0) 
     	obj := self first
     	self foreach(o, 
     		v1 := if(m, o doMessage(m, call sender), o)
     		v2 := if(m, obj doMessage(m, call sender), obj)
     		if(v1 < v2, obj := o)
     	)
     	obj
     )
     
  docSlot("flatten", """Creates a new list, with all contained lists flattened into the new list. For example:
<pre>
list(1,2,list(3,4,list(5))) flatten
==> list(1, 2, 3, 4, 5)
</pre>
  """)
  List flatten := method(
    l := List clone
    self foreach(v, if(v type == "List", l appendSeq(v flatten), l append(v)))
    l
  )
     
	docSlot("select(optionalIndex, value, message)", 
		   "Like foreach, but the values for which the result of 
message are non-Nil are returned in a new List. Example:
<pre>list(1, 5, 7, 2) select(i, v, v > 3) print
==> 5, 7

list(1, 5, 7, 2) select(v, v > 3) print
==> 5, 7</pre>")     
     
	docSlot("detect(optionalIndex, value, message)", 
		   "Returns the first value for which the message evaluates to a non-Nil. Example:
<pre>list(1, 2, 3, 4) detect(i, v, v > 2)
==> 3

list(1, 2, 3, 4) detect(v, v > 2)
==> 3</pre>")

	docSlot("map(optionalIndex, value, message)", "Same as calling mapInPlace() on a clone of the receiver, but more efficient.")
	
	docSlot("cusor", "Returns a ListCursor for the receiver.")
	cursor := method(ListCursor clone setCollection(self)) 
)

ListCursor := Object clone do(
	newSlot("index", 0)
	newSlot("collection")
	next := method(index = index + 1; if(index > (collection size - 1), index = collection size - 1))
	previous := method(index = index - 1; if(index < 0, index = 0))
	value := method(collection at(index))
)

