Sequence do(
	docSlot("repeated(aNumber)", "Returns a new sequence containing the receiver repeated aNumber number of times.")
	repeated := method(n,
		s := Sequence clone
		for(i, 0, n, s appendSeq(self))
		s 
	)
)

Call do(
	docSlot("description", "Returns a description of the receiver as a String.")

	description := method(
		m := self message
		s := self target type .. " " .. m name
		s .. " " repeated(s size - 35) .. m label lastPathComponent .. " " .. m lineNumber	
	)

	delegateTo := method(target, altSender,
		target doMessage(self message clone setNext setAttached, altSender ifNilEval(self sender))
	)

	evalArgs := method(
		self message argsEvaluatedIn(sender)
	)
)

Message description := method(
	self name .. " " repeated(self name size - 35) .. self label lastPathComponent .. " " .. self lineNumber	
)
	
FifoQueue := List clone do(pop := method(self removeFirst)) // not efficient

Scheduler := Object clone do(
	newSlot("yieldingCoros", FifoQueue clone)
	newSlot("timers", FifoQueue clone)
	currentCoroutine := method(Coroutine currentCoroutine)
)

Coroutine do(
	type := "Coroutine"
	newSlot("exception", nil)
	newSlot("parentCoroutine", nil)
	newSlot("runTarget", nil)
	newSlot("runLocals", nil)
	newSlot("runMessage", nil)
	newSlot("result", nil)
	newSlot("label", "")
	newSlot("inException", false)
	
	newSlot("yieldingCoros", List clone)
	
	label := method(self uniqueId)
	
	showYielding := method(s,
		writeln("   ", label, " ", s)
		yieldingCoros foreach(v, writeln("    ", v uniqueId))
	)
	
	resumeLater := method(
		yieldingCoros atInsert(0, self)
	)

	yield := method(
		//showYielding("yield")
		//writeln("Coro ", self uniqueId, " yielding - yieldingCoros = ", yieldingCoros size)
		yieldingCoros remove(self)
		yieldingCoros atInsert(0, self)
		next := yieldingCoros pop 
		//writeln("Coro ", next uniqueId, " resume")
		if(next, next resume)	
	)
	
	pause := method(
		//showYielding("pause")
		yieldingCoros remove(self)
		isCurrent ifTrue(
			next := yieldingCoros pop 
			if(next, 
				next resume, 
				writeln("Scheduler: nothing left to resume so we are exiting")
				System exit
			)
		)
	)
	
	yieldCurrentAndResumeSelf := method(
		//showYielding("yieldCurrentAndResumeSelf")
		yieldingCoros remove(self)
		isCurrent ifFalse(resume)
	)
	
	pauseCurrentAndResumeSelf := method(
		//showYielding("pauseCurrentAndResumeSelf")
		yieldingCoros remove(self)
		isCurrent ifFalse(resume)
	)

	//typeId := method(self type .. "_0x" .. self uniqueId asString toBase(16) .. " " .. self label)
	typeId := method(self type .. "_0x" .. self uniqueId asString toBase(16))

	callStack := method(
		stack := ioStack
		stack selectInPlace(v, Object argIsCall(getSlot("v"))) reverse 
		stack selectInPlace(v, (v target type == "Coroutine" and v message name == "setResult") not)  
		stack selectInPlace(v, (v target type == "Coroutine" and v message name == "main") not)  
		stack foreach(i, v, if(v target type == "Importer" and v message name == "find", stack sliceInPlace(i+1); break) )
		//stack foreach(v, writeln(v type)) 
		//System exit
		stack
	)
	
	backTraceString := method(
		if(Coroutine inException, 
			writeln("\nException: ", exception error, "\n\n")
			writeln("Coroutine Exception loop detected"); 
			System exit
		)
		Coroutine setInException(true)
		buf := Sequence clone
		
		if(CGI isInWebScript, buf appendSeq("<pre>"))
		
		if(exception, buf appendSeq("\n  Exception: ", exception error, "\n"))
		
		//buf appendSeq("  ", self typeId, " stack trace")
		
		if(callStack size > 0) then(
			buf appendSeq("  ---------\n")
			
			if(exception and exception caughtMessage, 
				buf appendSeq("  ", exception caughtMessage description, "\n")
			)
			
			callStack foreach(v, buf appendSeq("  ", v description, "\n"))
			buf appendSeq("\n")
		) else(buf appendSeq(" empty\n"))

		Coroutine setInException(false)
		buf
	)
	
	showStack := method(write(backTraceString))

	resumeParentCoroutine := method(
		if(parentCoroutine, parentCoroutine pauseCurrentAndResumeSelf)
	)

	main := method(
		setResult(runTarget doMessage(runMessage, runLocals))
		resumeParentCoroutine
	    //writeln("Coroutine error: attempt to return from Coroutine")
	    pause
	)

	raiseException := method(e, 
		self setException(e)
		resumeParentCoroutine
	)
	
)

Object wait := method(s, 
	endDate := Date clone now + Duration clone setSeconds(s)
	loop(endDate isPast ifTrue(break); yield)
)

Message do(
	codeOfLength := method(length,
		c := self code
		if (c size < length, c, c slice(0, length) .. "...") asMutable replaceSeq("\n", ";")
	)

	asStackEntry := method(
		label := label lastPathComponent fileName
		label .. " " repeated(label size - 18) .. lineNumber .. " " repeated(lineNumber asString size - 6)  .. name
	)
)

Object do(
	try := method(
		coro := Coroutine clone
		coro setParentCoroutine(Scheduler currentCoroutine) 
		coro setRunTarget(call sender) 
		coro setRunLocals(call sender)
		coro setRunMessage(call argAt(0)) 
		coro run
		if(coro exception, coro exception, nil)
	)
	
	coroDo := method(
		coro := Coroutine clone
		coro setRunTarget(self) 
		coro setRunLocals(call sender)
		coro setRunMessage(call argAt(0)) 
		Coroutine yieldingCoros atInsert(0, Scheduler currentCoroutine)
		coro run 
		coro
	)
	
	currentCoro := method(Coroutine currentCoroutine)
)

nil do(
	catch := nil
	pass := nil
)

Protos Exception do(
	type := "Exception"
	newSlot("error", nil)
	newSlot("coroutine", nil)
	newSlot("caughtMessage", nil)
	
	raise := method(error,
		coro := Scheduler currentCoroutine
		coro raiseException(self clone setError(error) setCoroutine(coro))
	)
	
	catch := method(exceptionProto,
		if (self isKindOf(exceptionProto),
			call evalArgAt(1)
		)
		self
	)

	pass := method(Scheduler currentCoroutine raiseException(self)) 

	showStack := method(coroutine showStack)
)
