Lobby IoVM do(
	ImmutableSequence := ""
	String := ImmutableSequence
)

Sequence do(

	setSlot("..", method(arg, self asString cloneAppendSeq(arg asString)))

    docSlot("alignLeftInPlace(width, [padding])", "Same as align left but operation is performed on the receiver.")
    alignLeftInPlace := method(width, padding,
        originalSize := size
        padding = padding ifNilEval(" ")
        ((width - size) / padding size) ceil repeatTimes(appendSeq(padding))
        setSize(width max(originalSize))
    )
    
    docSlot("removeSeq(aSequence)", "Removes occurances of aSequence from the receiver.")
    removeSeq := method(s, self replaceSeq(s, ""))

    docSlot("alignLeft(width, [padding])", """
    Example:
    <pre>
    Io> "abc" alignLeft(10, "-")
    ==> abc-------
    Io> "abc" alignLeft(10, "-=")
    ==> abc-=-=-=-
    </pre>
    """)
    alignLeft := method(width, padding,
        asMutable alignLeftInPlace(width, padding)
    )

    docSlot("alignRight(width, [padding]", """
    Example:
    <pre>
    Io> "abc" alignRight(10, "-")
    ==> -------abc
    Io> "abc" alignRight(10, "-=")
    ==> -=-=-=-abc
    </pre>
    """)
    alignRight := method(width, padding,
        Sequence clone alignLeftInPlace(width - size, padding) appendSeq(self)
    )

    docSlot("alignCenter(width, [padding]", """
    Example:
    <pre>
    Io> "abc" alignCenter(10, "-")
    ==> ---abc----
    Io> "abc" alignCenter(10, "-=")
    ==> -=-abc-=-=
    </pre>
    """)
    alignCenter := method(width, padding,
        alignRight(((size + width) / 2) floor, padding) alignLeftInPlace(width, padding)
    )
    
    asSimpleString := method("\"" .. self asString .. "\"")

	splitNoEmpties := method(
		docSlot("split(optionalArg1, optionalArg2, ...)", """
		Returns a list containing the non-empty sub-sequences of the receiver divided by the given arguments.
		If no arguments are given the sequence is split on white space.
		Examples:
		<pre>
		"a   b  c d" splitNoEmpties => list("a", "b", "c", "d")
		"a***b**c*d" splitNoEmpties("*") => list("a", "b", "c", "d")
		"a***b||c,d" splitNoEmpties("*", "|", ",") => list("a", "b", "c", "d")
		</pre>
		""")
	
		self doMessage(
			call message clone setName("split") setAttachedMessage(nil) setNextMessage(nil)
		,
			call sender
		) selectInPlace(s, s size != 0)
	)

  docSlot("findNthSeq(aSequence, n)", "Returns a number with the nth occurence of aSequence")
  Sequence findNthSeq := method(str, n,
    num := self findSeq(str)
    if(num isNil, return nil)
    if(n == 1, return num)
    num + self slice(num + 1, self size) findNthSeq(str, n - 1)
  ) 

  docSlot("interpolate", "Takes #<lang>{ ... } tags and expands them using an appropriate interpreter. Returns a new string with the values expanded. Optional argument is the context to evaluate the code into. Currently the only <lang>'s supported are \"io\".")
  Sequence interpolate := method(ctx,
		if(ctx isNil, ctx = call sender)
  		strings := List clone
		start := 0
		while (start < self size,
			s := start
			start := self findSeq("#io{", s)
			if(start == nil, strings append(self slice(s, self size)); break)
			strings append(self slice(s, start))
			
			end := self findSeq("}", start)
			if(end == nil, strings append(self slice(s, self size)); break)
			
			//writeln("start = ", start, " end = ", end)
			
			exp := self slice(start+4, end)
			strings append(ctx doMessage(exp asMessage, call sender) asString)
			start := end + 1
		)
		strings join
	)
	
  docSlot("interpolateInPlace", "Takes #<lang>{ ... } tags and expands them using an appropriate interpreter. Modifies and returns the string with the values expanded. Optional argument is the context to evaluate the code into. Currently the only <lang>'s supported are \"io\".")

	Sequence interpolateInPlace := method(ctx,
		if(self isMutable == false,
			Exception raise("'interpolateInPlace' cannot be called on an immutable Sequence")
		)
		if(ctx isNil, ctx = call sender)
		self copy(self interpolate(ctx))
	)
)
