
Protos CLI := Object clone do(
	type := "CLI"
	stdout := File clone standardOutput
	stdin  := File clone standardInput
	inPrompt  := "Io> "
	outPrompt := "==> "
	multiLine := true
	newSlot("evalContext", method(Lobby))
	newSlot("isRunning", true)
	
	stop := method(setIsRunning(false))
	
	runPath := method(path,
		//Lobby launchPath := path pathComponent
		Lobby launchPath := if(path beginsWithSeq("/") /* i.e. absolute */, path,
			Directory currentWorkingDirectory asMutable appendPathSeq(path)
		) pathComponent

		e := try(Lobby doFile(path))
		if(e, e showStack)
	)

	run := method(
		Lobby launchPath := Directory currentWorkingDirectory
        Lobby exit := method(System exit)
        
		if(?args and(args size > 0), 
			args foreach(i, arg,
			
				if(arg == "-e", 
					writeln(evalContext doString(args slice(i + 1) join(" ")))
					return
				)
				
				if(arg == "-i",
					if(args size == i + 1,
						if(File clone setPath("main.io") exists,
							runPath("main.io")
						)
					,
						nextArg := args at(i + 1) 
						if(nextArg, runPath(nextArg))
					)
					return interactiveMultiline
				)
				
				runPath(arg)
				return
			)
		,
			if(File clone setPath("main.io") exists,
				runPath("main.io")
				return
			)
		)

        interactiveMultiline
	)
	
	outputResult := method(result,
			writeln("\n", outPrompt, getSlot("result"))
	)
	
	interactive := method(
 		writeln(System distribution, " ", System version)
 		while(isRunning,
			write(inPrompt)
			line := stdin readLine
			if(stdin isAtEnd, writeln; Lobby exit)
			result := nil
			e := try(result = evalContext doString(line))
			if(e, e showStack)
			outputResult(getSlot("result"))
		)
	)
	
	interactiveMultiline := method(
		writeln(System distribution, " ", System version)
		errTriQuote	:= try(Compiler messageForString("\"\"\"")) error
		errParen		:= try(Compiler messageForString("(")) error
		errArg		:= try(Compiler messageForString("(x,")) error
		errParen2	:= try(Compiler messageForString("1+(2+(")) error
		mapContinueOnErr := Map clone
		mapContinueOnErr atPut(errTriQuote, "\"-> ")
		mapContinueOnErr atPut(errParen, ")-> ")
		mapContinueOnErr atPut(errParen2, "))> ")

		while(isRunning,
			prompt := inPrompt
			line := ""
			loop(
				write(prompt)
				nextLine := stdin readLine
				if(stdin isAtEnd, writeln; Lobby exit)
				line := line .. "\n" .. (nextLine)
				if(line endsWithSeq("""\\"""),
					prompt = "    "
					continue
				)
				result := nil
				e := try(result = evalContext doString(line))
				if(e,
					err := e error
					if(prompt = mapContinueOnErr at(err),
						//err println
						continue
					)
					if(err ==(errArg) and(line asMutable strip endsWithSeq(",")),
						//err println
						prompt = "a-> "
						continue
					)
					e showStack
				)
				break
			)
			outputResult(getSlot("result"))
		)
	)
)

Shell := Object clone do(
	newSlot("directory", Directory clone setPath("."))
	
	cd := method(name, 
		item := directory at(call argAt(0) name)
	    if(v type == "Directory", setDirectory(v))
	)
	
	ls := method(directory items foreach(name println))
	
	open := method(name,
		item := directory at(call argAt(0) name)
		if(item isUserExecutable) then(
			// insert stuff to add args
			System system(item path)
		)	
	)
	
	forward := method(
		call delegateTo(directory)
		/*
		item := directory at(call message name)
		if(item == nil, writeln("no such path"); return item)
		*/
	)
)

Lobby shell := method(
	CLI setEvalContext(Shell)
	//CLI outputResult := method(nil)
)

