
MapTest := UnitTest clone do(
    setUp := method(
        super(setUp)
        self exampleMap := Map clone atPut("a", "alpha")  atPut("b", "beta")
    )
    
    testClone := method(
        assertNotSame(Map, Map clone)
		assertEquals(2, exampleMap size)
		clonedMap := exampleMap clone
        assertNotSame(clonedMap, exampleMap)
        assertEquals(2, clonedMap size)
        assertEquals(exampleMap at("a"), clonedMap at("a"))
        assertEquals(exampleMap at("b"), clonedMap at("b"))
    )

    testEmpty := method(
        exampleMap empty
        assertEquals(0, exampleMap size)
    )
    
    testAt := method(
        assertRaisesException(exampleMap at())
        assertRaisesException(exampleMap at(Nil))
        assertEquals("beta", exampleMap at("b"))
        assertEquals("alpha", exampleMap at("a"))
        assertNil(exampleMap at("c"))
        assertNil(exampleMap at("alpha"))
    )
    
    testAtPut := method(
        assertRaisesException(exampleMap atPut(Nil, "thirtythree"))
        assertRaisesException(exampleMap atPut(33))
        assertRaisesException(exampleMap atPut(33, "thirtythree"))
        exampleMap atPut("a", 88)
        exampleMap atPut("blah", "fasel")
        exampleMap atPut("nil", Nil)
        assertEquals("fasel", exampleMap at("blah"))
        assertEquals(88, exampleMap at("a"))
        assertNil(exampleMap at("nil"))
    )
    
    testAtIfAbsentPut := method(
        assertRaisesException(exampleMap atIfAbsentPut(Nil, "thirtythree"))
        assertRaisesException(exampleMap atIfAbsentPut(33))
        assertRaisesException(exampleMap atIfAbsentPut(33, "thirtythree"))
        exampleMap atIfAbsentPut("a", 88)
        exampleMap atIfAbsentPut("blah", "fasel")
        assertEquals("fasel", exampleMap at("blah"))
        assertEquals("alpha", exampleMap at("a"))
    )
    
    testsize := method(
        assertEquals(0, Map size)
        assertEquals(2, exampleMap size)
        exampleMap atPut("blah", "fasel")
        assertEquals(3, exampleMap size)
    )
    
    testKeys := method(
        assertEquals(List type, exampleMap keys type)
        assertEquals(0, Map keys size)
        keys := exampleMap keys
        assertEquals(2, keys size)
        assertTrue(keys contains("a"))
        assertTrue(keys contains("b"))
    )
    
    testValues := method(
        assertEquals(List type, exampleMap values type)
        assertEquals(0, Map values size)
        exampleMap atPut("nil", Nil)
        values := exampleMap values
        assertEquals(3, values size)
        assertTrue(values contains("alpha"))
        assertTrue(values contains("beta"))
        assertTrue(values contains(Nil))
    )
    
    testForeach := method(
        s := ""
        exampleMap foreach(key, value, 
        	//writeln("key = ", key)
        	//writeln("value = ", value)
            s = s .. key .. value
        )
        //writeln("\ns = [", s, "]")
        assertTrue(s == "aalphabbeta" or s == "bbetaaalpha" )
    )
    
    testHasKey := method(
        assertRaisesException(exampleMap hasKey())
        assertRaisesException(exampleMap hasKey(Nil))
        assertTrue(exampleMap hasKey("a"))
        assertTrue(exampleMap hasKey("b"))
        assertFalse(exampleMap hasKey("c"))
    )
    
    testHasValue := method(
        assertTrue(exampleMap hasValue("alpha"))
        assertTrue(exampleMap hasValue("beta"))
        assertFalse(exampleMap hasValue("whatever"))
    )
    
    testRemoveAt := method(
        assertRaisesException(exampleMap removeAt())
        assertRaisesException(exampleMap removeAt(Nil))
        exampleMap removeAt("c")
        assertEquals(2, exampleMap size)
        exampleMap removeAt("a")
        assertEquals(1, exampleMap size)
        //assertFalse(exampleMap hasKey("a"))
    )
)
